@@ -16,6 +16,7 @@ from __future__ import annotations
 
 import numpy as np
 from beartype.typing import Any, Mapping, Sequence
+from typing_extensions import override
 
 from superlinked.framework.common.exception import (
     MismatchingDimensionException,
@@ -25,39 +26,42 @@ from superlinked.framework.common.schema.blob_information import BlobInformation
 from superlinked.framework.common.schema.image_data import ImageData
 
 Json = Mapping[str, Any]
-NPArray = np.ndarray[
-    Any,
-    np.dtype[np.float64],  # type: ignore # numpy stub is missing for mypy-pylance
-]
+VectorItemT = np.float32
+NPArray = np.ndarray[VectorItemT]
 NP_PRINT_PRECISION = 6
 
 
 class Vector:
+
     def __init__(
         self,
-        value: Sequence[float] | Sequence[np.float64] | NPArray,
+        value: Sequence[float] | Sequence[np.floating] | NPArray,
         negative_filter_indices: set[int] | frozenset[int] | None = None,
-        vector_before_normalization: Vector | None = None,
+        denormalizer: float = 1.0,
     ) -> None:
         if isinstance(value, np.ndarray):
-            value_to_set = value
+            value_to_set = value.astype(VectorItemT, copy=False)
         else:
-            value_to_set = np.array(list(value), dtype=np.float64)
+            value_to_set = np.array(list(value), dtype=VectorItemT)
         self.value: NPArray = value_to_set
+        self.__make_value_immutable()
         self.__dimension: int = len(self.value)
         self.__negative_filter_indices = (
             frozenset(negative_filter_indices) if negative_filter_indices else frozenset({})
         )
         self.__validate_negative_filter_indices()
-        self.__vector_before_normalization = vector_before_normalization
+        self.__denormalizer = denormalizer
+
+    def __make_value_immutable(self) -> None:
+        self.value.setflags(write=False)
 
     @property
     def dimension(self) -> int:
         return self.__dimension
 
     @property
-    def vector_before_normalization(self) -> Vector | None:
-        return self.__vector_before_normalization
+    def denormalizer(self) -> float:
+        return self.__denormalizer
 
     @staticmethod
     def empty_vector() -> Vector:
@@ -76,8 +80,10 @@ class Vector:
         return self.dimension == 0
 
     @property
-    def without_negative_filter(self) -> Vector:
-        return Vector(self.value[self.non_negative_filter_mask])
+    def value_without_negative_filter(self) -> NPArray:
+        if self.__negative_filter_indices:
+            return self.value[self.non_negative_filter_mask]
+        return self.value
 
     @property
     def non_negative_filter_mask(self) -> np.ndarray:
@@ -86,30 +92,24 @@ class Vector:
         return mask
 
     def normalize(self, length: float) -> Vector:
-        if length in [0, 1]:
-            return self.copy_with_new(vector_before_normalization=self)
-        if self.is_empty:
+        if length in [0, 1] or self.is_empty:
             return self
-        normalized = (
-            self.copy_with_new(
-                self.without_negative_filter.value,
-                set(),
-                self,
-            )
-            / length
-        )
+        normalized = self.shallow_copy_with_new(self.value_without_negative_filter / float(length), set(), 1 / length)
         return normalized.apply_negative_filter(self)
 
+    def denormalize(self) -> Vector:
+        return self.normalize(self.denormalizer)
+
     def aggregate(self, vector: Vector) -> Vector:
         if self.is_empty:
-            return vector.__copy()
+            return vector
         if vector.is_empty:
-            return self.__copy()
+            return self
         if self.dimension != vector.dimension:
             raise MismatchingDimensionException(
                 f"Cannot aggregate vectors with different dimensions: {self.dimension} != {vector.dimension}"
             )
-        return self.copy_with_new(
+        return self.shallow_copy_with_new(
             self.value + vector.value,
             negative_filter_indices=self.negative_filter_indices.intersection(vector.negative_filter_indices),
         )
@@ -131,46 +131,24 @@ class Vector:
     def apply_negative_filter(self, other: Vector) -> Vector:
         if self.negative_filter_indices == other.negative_filter_indices:
             values = [
-                (self.value[i] if i not in other.negative_filter_indices else value)
-                for i, value in enumerate(other.value)
+                (value if i in other.negative_filter_indices else self.value[i]) for i, value in enumerate(other.value)
             ]
         else:
             value_iterator = iter(self.value)
             values = [
-                (next(value_iterator) if i not in other.negative_filter_indices else value)
+                (value if i in other.negative_filter_indices else next(value_iterator))
                 for i, value in enumerate(other.value)
             ]
-        return self.copy_with_new(values, other.negative_filter_indices)
+        return self.shallow_copy_with_new(values, other.negative_filter_indices)
 
     def replace_negative_filters(self, new_negative_filter_value: float) -> Vector:
-        return self.copy_with_new(
+        return self.shallow_copy_with_new(
             [
                 (new_negative_filter_value if i in self.negative_filter_indices else original_value)
                 for i, original_value in enumerate(self.value)
             ]
         )
 
-    def concatenate(self, other: Any) -> Vector:
-        if not isinstance(other, Vector):
-            return NotImplemented
-        if self.is_empty:
-            return other.__copy()
-        if other.is_empty:
-            return self.__copy()
-        negative_filter_indices = self.negative_filter_indices.union(
-            {i + self.dimension for i in other.negative_filter_indices}
-        )
-        vector_before_normalization = (
-            self.vector_before_normalization.concatenate(other.vector_before_normalization)
-            if self.vector_before_normalization and other.vector_before_normalization
-            else None
-        )
-        return self.copy_with_new(
-            np.concatenate((self.value, np.array(other.value, dtype=np.float64))),
-            negative_filter_indices,
-            vector_before_normalization,
-        )
-
     def split(self, lengths: list[int]) -> list[Vector]:
         if sum(lengths) < self.dimension:
             raise ValueError(
@@ -185,30 +163,37 @@ class Vector:
             negative_filter_indices = {
                 idx - start_index for idx in self.negative_filter_indices if start_index <= idx < end_index
             }
-            split_vectors.append(Vector(split_values[i], negative_filter_indices, None))
+            split_vectors.append(Vector(split_values[i], negative_filter_indices))
         return split_vectors
 
-    def __mul__(self, other: float | int | Vector) -> Vector:
+    def __mul__(self, other: Any) -> Vector:
         if self.is_empty:
             return self
-        if isinstance(other, int | float):
-            return (
-                self.copy_with_new(self.value) if float(other) == 1.0 else self.copy_with_new(self.value * float(other))
+        if not isinstance(other, int | float | np.floating):
+            raise ValueError(f"{type(self).__name__} can only be multiplied with int or float")
+        if other == 1:
+            return self
+        if other == 0:
+            return Vector(
+                np.zeros(
+                    self.dimension,
+                    dtype=VectorItemT,  # type: ignore[arg-type] # it is valid
+                ),
+                self.negative_filter_indices,
             )
-        if self.dimension != other.dimension:
-            raise ValueError(
-                f"Vector dimensions are not equal. First Vector dimension={self.dimension} "
-                f"other Vector dimension={other.dimension}"
-            )
-        return self.copy_with_new(self.value * other.value)
+
+        multiplied_vector = Vector(self.value_without_negative_filter * float(other))
+        return multiplied_vector.apply_negative_filter(self)
+
+    def __rmul__(self, other: Any) -> Vector:
+        return self * other
 
     def __truediv__(self, other: Any) -> Vector:
-        if (not isinstance(other, (float, int))) or other == 0:
+        if other == 0:
             return NotImplemented
-        if self.is_empty:
-            return self
-        return self.copy_with_new(self.value / float(other))
+        return self * (1 / other)
 
+    @override
     def __eq__(self, other: Any) -> bool:
         if isinstance(other, Vector):
             return (
@@ -217,40 +202,35 @@ class Vector:
             )
         return False
 
+    @override
     def __hash__(self) -> int:
         return hash((str(self.value), self.negative_filter_indices))
 
-    def copy_with_new(
+    def shallow_copy_with_new(
         self,
-        value: list[float] | list[np.float64] | NPArray | None = None,
+        value: list[float] | list[VectorItemT] | list[np.floating] | NPArray | None = None,
         negative_filter_indices: set[int] | frozenset[int] | None = None,
-        vector_before_normalization: Vector | None = None,
+        denormalizer: float | None = None,
     ) -> Vector:
         value_to_use = self.value if value is None else value
-        vector_before_normalization_to_use = (
-            (self.vector_before_normalization.copy_with_new() if self.vector_before_normalization is not None else None)
-            if vector_before_normalization is None
-            else vector_before_normalization.copy_with_new()
-        )
         negative_filter_indices_to_use = (
             self.negative_filter_indices if negative_filter_indices is None else negative_filter_indices
         )
-        return Vector(
-            value_to_use.copy(),
-            negative_filter_indices_to_use.copy(),
-            vector_before_normalization_to_use,
-        )
+        denormalizer_to_use = self.denormalizer if denormalizer is None else denormalizer
+        return Vector(value_to_use, negative_filter_indices_to_use, denormalizer_to_use)
 
-    def __copy(self) -> Vector:
-        if self.is_empty:
-            return self
-        return self.copy_with_new()
+    def to_list(self) -> list[float]:
+        return [float(x) for x in self.value.tolist()]
 
     def __str__(self) -> str:
         return np.array_str(  # type: ignore # numpy stub is missing for mypy-pylance
             self.value, precision=NP_PRINT_PRECISION, suppress_small=True
         )
 
+    @override
+    def __repr__(self) -> str:
+        return self.__str__()
+
     @classmethod
     def init_zero_vector(cls, length: int) -> Vector:
         return Vector([0] * length)
