@@ -16,9 +16,11 @@ from beartype.typing import Any, Sequence, cast
 
 from superlinked.framework.common.data_types import NodeDataTypes, Vector
 from superlinked.framework.common.schema.blob_information import BlobInformation
+from superlinked.framework.common.schema.id_field import IdField
 from superlinked.framework.common.schema.image_data import ImageData
 from superlinked.framework.common.schema.schema_object import (
     Blob,
+    Boolean,
     ConcreteSchemaField,
     Float,
     FloatList,
@@ -31,20 +33,23 @@ from superlinked.framework.common.schema.schema_object import (
 from superlinked.framework.common.storage.field.field_data_type import FieldDataType
 from superlinked.framework.common.util.generic_class_util import GenericClassUtil
 
-FIELD_DATA_TYPE_BY_SCHEMA_FIELD_TYPE: dict[type[ConcreteSchemaField], FieldDataType] = {
+FIELD_DATA_TYPE_BY_SCHEMA_FIELD_TYPE: dict[type[ConcreteSchemaField | IdField], FieldDataType] = {
     Blob: FieldDataType.BLOB,
     FloatList: FieldDataType.FLOAT_LIST,
     StringList: FieldDataType.STRING_LIST,
     Float: FieldDataType.DOUBLE,
     Integer: FieldDataType.INT,
+    Boolean: FieldDataType.BOOLEAN,
     String: FieldDataType.STRING,
     Timestamp: FieldDataType.INT,
+    IdField: FieldDataType.METADATA_STRING,
 }
 
 FIELD_DATA_TYPE_BY_NODE_DATA_TYPE: dict[type[NodeDataTypes | dict], FieldDataType] = {
     dict: FieldDataType.JSON,
     float: FieldDataType.DOUBLE,
     int: FieldDataType.INT,
+    bool: FieldDataType.BOOLEAN,
     str: FieldDataType.STRING,
     list[float]: FieldDataType.FLOAT_LIST,
     list[str]: FieldDataType.STRING_LIST,
@@ -57,10 +62,12 @@ VALID_TYPE_BY_FIELD_DATA_TYPE: dict[FieldDataType, Sequence[type[NodeDataTypes |
     FieldDataType.BLOB: [BlobInformation],
     FieldDataType.DOUBLE: [int, float],
     FieldDataType.INT: [int],
+    FieldDataType.BOOLEAN: [bool],
     FieldDataType.JSON: [dict],
     FieldDataType.FLOAT_LIST: [list[float]],
     FieldDataType.STRING_LIST: [list[str]],
     FieldDataType.STRING: [str],
+    FieldDataType.METADATA_STRING: [str],
     FieldDataType.VECTOR: [Vector],
     FieldDataType.IMAGE_DATA: [ImageData],
 }
