@@ -27,29 +27,35 @@ from superlinked.framework.common.storage.index_config import IndexConfig
 from superlinked.framework.common.storage.query.vdb_knn_search_params import (
     VDBKNNSearchParams,
 )
+from superlinked.framework.common.storage.vdb_connector import VDBKNNSearchConfigT
 
 KNNReturnT = TypeVar("KNNReturnT")
 SearchParamsT = TypeVar("SearchParamsT", bound=VDBKNNSearchParams)
 QuertT = TypeVar("QuertT")
 
 
-class Search(ABC, Generic[SearchParamsT, QuertT, KNNReturnT]):
-    def knn_search_with_checks(
+class Search(ABC, Generic[SearchParamsT, QuertT, KNNReturnT, VDBKNNSearchConfigT]):
+    async def knn_search_with_checks(
         self,
         index_config: IndexConfig,
         search_params: SearchParamsT,
+        search_config: VDBKNNSearchConfigT,
     ) -> KNNReturnT:
         self.check_vector_field(index_config, search_params.vector_field)
         self.check_filters(index_config, search_params.filters)
-        query = self.build_query(search_params)
-        return self.knn_search(index_config, query)
+        query = self.build_query(search_params, search_config)
+        return await self.knn_search(index_config, query)
 
     @abstractmethod
-    def build_query(self, search_params: SearchParamsT) -> QuertT:
+    def build_query(
+        self,
+        search_params: SearchParamsT,
+        search_config: VDBKNNSearchConfigT,
+    ) -> QuertT:
         pass
 
     @abstractmethod
-    def knn_search(self, index_config: IndexConfig, query: QuertT) -> KNNReturnT:
+    async def knn_search(self, index_config: IndexConfig, query: QuertT) -> KNNReturnT:
         pass
 
     @staticmethod
@@ -69,7 +75,10 @@ class Search(ABC, Generic[SearchParamsT, QuertT, KNNReturnT]):
     ) -> None:
         if not filters:
             return
-        if unindexed_filters := [
-            filter_ for filter_ in filters if cast(Field, filter_._operand).name not in index_config.indexed_field_names
+        filter_field_names = [cast(Field, filter_._operand).name for filter_ in filters]
+        if unindexed_filter_field_names := [
+            filter_field_name
+            for filter_field_name in filter_field_names
+            if filter_field_name not in index_config.indexed_field_names
         ]:
-            raise ValidationException(f"Unindexed filters found: {unindexed_filters}")
+            raise ValidationException(f"Unindexed fields with filter found: {unindexed_filter_field_names}")
