@@ -17,7 +17,7 @@ from dataclasses import dataclass
 from beartype.typing import Any, Mapping, Sequence, TypeVar, cast
 
 from superlinked.framework.common.dag.index_node import IndexNode
-from superlinked.framework.common.data_types import NodeDataTypes, PythonTypes, Vector
+from superlinked.framework.common.data_types import PythonTypes, Vector
 from superlinked.framework.common.exception import InvalidSchemaException
 from superlinked.framework.common.interface.comparison_operand import (
     ComparisonOperation,
@@ -29,6 +29,7 @@ from superlinked.framework.common.schema.schema_object import (
     SchemaField,
     SchemaObject,
 )
+from superlinked.framework.common.storage.entity.entity import Entity
 from superlinked.framework.common.storage.entity.entity_data import EntityData
 from superlinked.framework.common.storage.field.field import Field
 from superlinked.framework.common.storage.field.field_data import (
@@ -53,10 +54,13 @@ from superlinked.framework.common.storage_manager.entity_builder import EntityBu
 from superlinked.framework.common.storage_manager.knn_search_params import (
     KNNSearchParams,
 )
+from superlinked.framework.common.storage_manager.node_result_data import NodeResultData
 from superlinked.framework.common.storage_manager.search_result_item import (
     SearchResultItem,
 )
 from superlinked.framework.common.storage_manager.storage_naming import StorageNaming
+from superlinked.framework.common.util.async_util import AsyncUtil
+from superlinked.framework.dsl.query.query_user_config import QueryUserConfig
 
 ResultTypeT = TypeVar("ResultTypeT")
 # NodeDataValueType
@@ -79,8 +83,8 @@ class StorageManager:
         self._storage_naming = StorageNaming()
         self._entity_builder = EntityBuilder(self._storage_naming)
 
-    def close_connection(self) -> None:
-        self._vdb_connector.close_connection()
+    async def close_connection(self) -> None:
+        await self._vdb_connector.close_connection()
 
     def init_search_indices(
         self,
@@ -102,11 +106,12 @@ class StorageManager:
             index_field_descriptors,
         )
 
-    def knn_search(
+    async def knn_search(
         self,
         index_node: IndexNode,
         schema: IdSchemaObject,
         knn_search_params: KNNSearchParams,
+        query_user_config: QueryUserConfig,
         should_return_index_vector: bool = False,
         **params: Any,
     ) -> Sequence[SearchResultItem]:
@@ -120,7 +125,8 @@ class StorageManager:
         fields_to_return = list(schema_fields_by_fields.keys()) + list(self._entity_builder._admin_fields.header_fields)
         if should_return_index_vector:
             fields_to_return.append(self._entity_builder.compose_field(index_node.node_id, Vector))
-        search_result: Sequence[ResultEntityData] = self._vdb_connector.knn_search(
+        search_config = self._vdb_connector.init_search_config(query_user_config)
+        search_result: Sequence[ResultEntityData] = await self._vdb_connector.knn_search(
             index_name,
             schema._schema_name,
             VDBKNNSearchParams(
@@ -130,6 +136,7 @@ class StorageManager:
                 self._compose_filter_field_data(schema, knn_search_params.filters),
                 knn_search_params.radius,
             ),
+            search_config,
             **params,
         )
         schema_field_by_field_name = self._create_schema_field_by_field_name(schema_fields_by_fields)
@@ -208,65 +215,44 @@ class StorageManager:
             self._entity_builder.compose_entity_data_from_parsed_schema(parsed_schema)
             for parsed_schema in parsed_schemas
         ]
-        self._vdb_connector.write_entities(entities_to_write)
+        AsyncUtil.run(self._vdb_connector.write_entities(entities_to_write))
 
-    def write_node_result(
-        self,
-        schema: SchemaObject,
-        object_id: str,
-        node_id: str,
-        result: NodeDataTypes,
-        origin_id: str | None = None,
-    ) -> None:
-        entity_id = self._entity_builder.compose_entity_id(schema._schema_name, object_id)
-        field_data = list(self._entity_builder._admin_fields.create_header_field_data(entity_id, origin_id))
-        if result is not None:
-            result_field_data = self._entity_builder.compose_field_data(node_id, result)
-            field_data.append(result_field_data)
-        self._vdb_connector.write_entities([EntityData(entity_id, {fd.name: fd for fd in field_data})])
+    def write_node_results(self, node_data_items: Sequence[NodeResultData]) -> None:
+        entities_to_write = [
+            self._entity_builder.compose_entity_data_from_node_result(node_data) for node_data in node_data_items
+        ]
+        AsyncUtil.run(self._vdb_connector.write_entities(entities_to_write))
 
     def write_node_data(
-        self,
-        schema: SchemaObject,
-        object_id: str,
-        node_id: str,
-        node_data: dict[str, PythonTypes],
+        self, schema: SchemaObject, node_data_by_object_id: Mapping[str, dict[str, PythonTypes]], node_id: str
     ) -> None:
+        entity_data_items = [
+            self._compose_entity_data(schema, object_id, node_id, node_data)
+            for object_id, node_data in node_data_by_object_id.items()
+        ]
+        AsyncUtil.run(self._vdb_connector.write_entities(entity_data_items))
+
+    def _compose_entity_data(
+        self, schema: SchemaObject, object_id: str, node_id: str, node_data: dict[str, PythonTypes]
+    ) -> EntityData:
         entity_id = self._entity_builder.compose_entity_id(schema._schema_name, object_id)
         field_data = list(self._entity_builder._admin_fields.create_header_field_data(entity_id))
         field_data.extend(list(self._entity_builder.compose_field_data_from_node_data(node_id, node_data)))
-        self._vdb_connector.write_entities([EntityData(entity_id, {fd.name: fd for fd in field_data})])
-
-    def read_schema_field_values(self, object_id: str, schema_fields: Sequence[SchemaField]) -> ParsedSchema:
-        schema = self._get_schema_of_schema_fields(schema_fields)
-        entity_id = self._entity_builder.compose_entity_id(schema._schema_name, object_id)
-        fields = {
-            self._entity_builder.convert_schema_field_to_field(schema_field): schema_field
-            for schema_field in schema_fields
-        }
-        entity = self._entity_builder.compose_entity(entity_id, list(fields.keys()))
-        entity_data = self._vdb_connector.read_entities([entity])[0]
-
-        schema_field_by_field_name = self._create_schema_field_by_field_name(fields)
-        return self._entity_builder.parse_entity_data(schema, entity_data, schema_field_by_field_name)
-
-    def _get_schema_of_schema_fields(
-        self,
-        schema_fields: Sequence[SchemaField],
-    ) -> IdSchemaObject:
-        schemas = {schema_field.schema_obj for schema_field in schema_fields}
-        if len(schemas) != 1:
-            raise InvalidSchemaException(f"`schema_fields` must have the same root schema, got {schemas}")
-        schema = schemas.pop()
-        if not isinstance(schema, IdSchemaObject):
-            raise InvalidSchemaException(f"The root schema of `schema_fields` must be an {IdSchemaObject.__name__}")
-        return schema
+        return EntityData(entity_id, {fd.name: fd for fd in field_data})
 
     def read_node_results(
         self,
         schemas_with_object_ids: Sequence[tuple[SchemaObject, str]],
         node_id: str,
         result_type: type[ResultTypeT],
+    ) -> list[ResultTypeT | None]:
+        return AsyncUtil.run(self.read_node_results_async(schemas_with_object_ids, node_id, result_type))
+
+    async def read_node_results_async(
+        self,
+        schemas_with_object_ids: Sequence[tuple[SchemaObject, str]],
+        node_id: str,
+        result_type: type[ResultTypeT],
     ) -> list[ResultTypeT | None]:
         entity_ids = [
             self._entity_builder.compose_entity_id(schema._schema_name, object_id)
@@ -274,43 +260,50 @@ class StorageManager:
         ]
         result_field = self._entity_builder.compose_field(node_id, result_type)
         entities = [self._entity_builder.compose_entity(entity_id, [result_field]) for entity_id in entity_ids]
-        entity_data = self._vdb_connector.read_entities(entities)
+        entity_data = await self._vdb_connector.read_entities(entities)
 
         def cast_value_if_not_none(field_data: FieldData | None) -> ResultTypeT | None:
             return cast(ResultTypeT, field_data.value) if field_data else None
 
         return [cast_value_if_not_none(ed.field_data.get(result_field.name)) for ed in entity_data]
 
-    def read_node_result(
+    async def read_node_result(
         self,
         schema: SchemaObject,
         object_id: str,
         node_id: str,
         result_type: type[ResultTypeT],
     ) -> ResultTypeT | None:
-        return next(iter(self.read_node_results([(schema, object_id)], node_id, result_type)))
+        return next(iter(await self.read_node_results_async([(schema, object_id)], node_id, result_type)))
 
     # TODO FAI-2737 to solve the parameters
     def read_node_data(
         self,
         schema: SchemaObject,
-        object_id: str,
+        object_ids: Sequence[str],
         node_id: str,
         node_data_descriptor: dict[str, type[NDVT]],
-    ) -> dict[str, NDVT]:
-        entity_id = self._entity_builder.compose_entity_id(schema._schema_name, object_id)
+    ) -> dict[str, dict[str, NDVT]]:
         field_by_node_data_key = self._map_fields_by_node_data_keys(node_id, node_data_descriptor or {})
         if not field_by_node_data_key:
-            return {}
-        fields = list(field_by_node_data_key.values())
+            return {object_id: {} for object_id in object_ids}
         node_data_key_by_field_name = {field.name: key for key, field in field_by_node_data_key.items()}
-        entity = self._entity_builder.compose_entity(entity_id, fields)
-        entity_data = self._vdb_connector.read_entities([entity])[0]
-        node_data = {
-            node_data_key_by_field_name[field_data.name]: cast(NDVT, field_data.value)
-            for field_data in entity_data.field_data.values()
+        fields = list(field_by_node_data_key.values())
+        entities = [self._compose_entity(schema, object_id, fields) for object_id in object_ids]
+        entity_data_items = AsyncUtil.run(self._vdb_connector.read_entities(entities))
+        node_data_items = {
+            object_id: {
+                node_data_key_by_field_name[field_data.name]: cast(NDVT, field_data.value)
+                for field_data in entity_data.field_data.values()
+            }
+            for object_id, entity_data in zip(object_ids, entity_data_items)
         }
-        return node_data
+        return node_data_items
+
+    def _compose_entity(self, schema: SchemaObject, object_id: str, fields: Sequence[Field]) -> Entity:
+        entity_id = self._entity_builder.compose_entity_id(schema._schema_name, object_id)
+        entity = self._entity_builder.compose_entity(entity_id, fields)
+        return entity
 
     def _compile_indexed_fields_to_descriptors(
         self, params: SearchIndexParams
@@ -320,7 +313,7 @@ class StorageManager:
             params.length,
             self._vdb_connector.distance_metric,
             self._vdb_connector.search_algorithm,
-            self._vdb_connector.vector_coordinate_type,
+            self._vdb_connector.vector_precision,
         )
 
         return (
@@ -328,7 +321,7 @@ class StorageManager:
             list(self._create_index_field_descriptors_from_schema_fields(params.indexed_fields))
             + [
                 IndexFieldDescriptor(
-                    FieldDataType.STRING,
+                    FieldDataType.METADATA_STRING,
                     self._entity_builder._admin_fields.schema_id.field.name,
                 )
             ],
