@@ -21,6 +21,7 @@ from superlinked.framework.dsl.executor.rest.rest_configuration import (
     RestQuery,
 )
 from superlinked.framework.dsl.query.query_mixin import QueryMixin
+from superlinked.framework.dsl.query.query_user_config import QueryUserConfig
 from superlinked.framework.dsl.query.result import QueryResult
 from superlinked.framework.dsl.source.rest_source import RestSource
 
@@ -59,11 +60,13 @@ class RestHandler:
         source = self.__path_to_source_map[path]
         source.put([input_schema])
 
-    def _query_handler(self, query_descriptor: dict, path: str, include_metadata: bool = False) -> QueryResult:
+    async def _query_handler(
+        self, query_descriptor: dict, path: str, query_user_config: QueryUserConfig
+    ) -> QueryResult:
         query = self.__path_to_query_map[path].query_descriptor
-        if include_metadata:
-            query = query.include_metadata()
-        return self.__query_mixin.query(query, **query_descriptor)
+        query = query.replace_user_config(query_user_config)
+        result = await self.__query_mixin.async_query(query, **query_descriptor)
+        return result
 
     def __create_path_to_resource_mapping(
         self,
