@@ -16,6 +16,7 @@ from pathlib import Path
 
 import structlog
 from beartype.typing import Sequence
+from PIL.Image import Image
 from typing_extensions import override
 
 from superlinked.framework.common.dag.embedding_node import EmbeddingNode
@@ -43,7 +44,16 @@ from superlinked.framework.common.space.config.normalization.normalization_confi
 from superlinked.framework.common.space.config.transformation_config import (
     TransformationConfig,
 )
-from superlinked.framework.common.space.embedding.image_embedding import ImageEmbedding
+from superlinked.framework.common.space.embedding.model_based.engine.embedding_engine_config import (
+    EmbeddingEngineConfig,
+)
+from superlinked.framework.common.space.embedding.model_based.engine.modal_engine_config import (
+    ModalEngineConfig,
+)
+from superlinked.framework.common.space.embedding.model_based.singleton_embedding_engine_manager import (
+    SingletonEmbeddingEngineManager,
+)
+from superlinked.framework.common.util.async_util import AsyncUtil
 from superlinked.framework.dsl.space.exception import InvalidSpaceParamException
 from superlinked.framework.dsl.space.image_space_field_set import (
     ImageDescriptionSpaceFieldSet,
@@ -68,6 +78,10 @@ class ImageSpace(Space[Vector, ImageData]):
             Defaults to "clip-ViT-B-32".
         model_handler (ModelHandler, optional): The handler for the model,
             defaults to ModelHandler.SENTENCE_TRANSFORMERS.
+        model_cache_dir (Path | None, optional): Directory to cache downloaded models.
+            If None, uses the default cache directory. Defaults to None.
+        embedding_engine_config (EmbeddingEngineConfig, optional): Configuration for the embedding engine.
+            Defaults to EmbeddingEngineConfig().
 
     Raises:
         InvalidSpaceParamException: If the image and description fields are not
@@ -78,8 +92,9 @@ class ImageSpace(Space[Vector, ImageData]):
         self,
         image: Blob | DescribedBlob | None | Sequence[Blob | DescribedBlob | None],
         model: str = "clip-ViT-B-32",
-        model_handler: ModelHandler = ModelHandler.SENTENCE_TRANSFORMERS,
         model_cache_dir: Path | None = None,
+        model_handler: ModelHandler = ModelHandler.SENTENCE_TRANSFORMERS,
+        embedding_engine_config: EmbeddingEngineConfig | None = None,
     ) -> None:
         """
         Initialize the ImageSpace instance for generating vector representations
@@ -94,22 +109,35 @@ class ImageSpace(Space[Vector, ImageData]):
                 defaults to ModelHandler.SENTENCE_TRANSFORMERS.
             model_cache_dir (Path | None, optional): Directory to cache downloaded models.
                 If None, uses the default cache directory. Defaults to None.
+            embedding_engine_config (EmbeddingEngineConfig, optional): Configuration for the embedding engine.
+                Defaults to EmbeddingEngineConfig().
 
         Raises:
             InvalidSpaceParamException: If the image and description fields are not
                 from the same schema.
         """
+        if embedding_engine_config is None:
+            embedding_engine_config = EmbeddingEngineConfig()
         non_none_image: list[Blob | DescribedBlob] = self._fields_to_non_none_sequence(image)
         self.__validate_field_schemas(non_none_image)
+        self.__validate_model_handler(model_handler, embedding_engine_config)
         image_fields, description_fields = self._split_images_from_descriptions(non_none_image)
         super().__init__(image_fields, Blob)
-        length = ImageEmbedding.init_manager(model_handler, model, model_cache_dir).calculate_length()
-        self.image = ImageSpaceFieldSet(self, set(image_fields))
+        length = AsyncUtil.run(
+            SingletonEmbeddingEngineManager().calculate_length(
+                model_handler, model, model_cache_dir, embedding_engine_config
+            )
+        )
+        self.image = ImageSpaceFieldSet(self, set(image_fields), allowed_param_types=[str, Image])
         self.description = ImageDescriptionSpaceFieldSet(
-            self, set(description for description in description_fields if description is not None)
+            self,
+            set(description for description in description_fields if description is not None),
+            allowed_param_types=[str],
         )
         self._all_fields = self.image.fields | self.description.fields
-        self._transformation_config = self._init_transformation_config(model, model_cache_dir, model_handler, length)
+        self._transformation_config = self._init_transformation_config(
+            model, model_cache_dir, model_handler, length, embedding_engine_config
+        )
         self.__embedding_node_by_schema = self._init_embedding_node_by_schema(
             image_fields, description_fields, self._all_fields, self.transformation_config
         )
@@ -123,6 +151,17 @@ class ImageSpace(Space[Vector, ImageData]):
         ):
             raise InvalidSpaceParamException("ImageSpace image and description field must be in the same schema.")
 
+    def __validate_model_handler(
+        self, model_handler: ModelHandler, embedding_engine_config: EmbeddingEngineConfig
+    ) -> None:
+        if model_handler == ModelHandler.MODAL and not isinstance(embedding_engine_config, ModalEngineConfig):
+            raise ValueError(
+                (
+                    f"When using {ModelHandler.MODAL} as model_handler, embedding_engine_config must "
+                    f"be an instance of ModalEngineConfig, but got {type(embedding_engine_config).__name__}"
+                )
+            )
+
     def _split_images_from_descriptions(
         self, images: Sequence[Blob | DescribedBlob]
     ) -> tuple[list[Blob], list[String | None]]:
@@ -168,14 +207,20 @@ class ImageSpace(Space[Vector, ImageData]):
         return False
 
     def _init_transformation_config(
-        self, model: str, model_cache_dir: Path | None, model_handler: ModelHandler, length: int
+        self,
+        model: str,
+        model_cache_dir: Path | None,
+        model_handler: ModelHandler,
+        length: int,
+        embedding_engine_config: EmbeddingEngineConfig,
     ) -> TransformationConfig[Vector, ImageData]:
         embedding_config = ImageEmbeddingConfig(
             ImageData,
             model,
             model_cache_dir,
-            model_handler,
             length,
+            embedding_engine_config,
+            model_handler,
         )
         aggregation_config = VectorAggregationConfig(Vector)
         normalization_config = L2NormConfig()
@@ -197,3 +242,16 @@ class ImageSpace(Space[Vector, ImageData]):
             )
             for image_field, description_field in zip(image_fields, description_fields)
         }
+
+    @override
+    def __eq__(self, other: object) -> bool:
+        return (
+            isinstance(other, ImageSpace)
+            and self.transformation_config == other.transformation_config
+            and self.image.fields == other.image.fields
+            and self.description.fields == other.description.fields
+        )
+
+    @override
+    def __hash__(self) -> int:
+        return hash((self.transformation_config, frozenset(self.image.fields), frozenset(self.description.fields)))
