@@ -14,6 +14,7 @@
 
 from beartype.typing import Any
 
+from superlinked.framework.common.precision import Precision
 from superlinked.framework.dsl.storage.vector_database import VectorDatabase
 from superlinked.framework.storage.common.vdb_settings import VDBSettings
 from superlinked.framework.storage.mongo_db.mongo_db_connection_params import (
@@ -43,6 +44,7 @@ class MongoDBVectorDatabase(VectorDatabase[MongoDBVDBConnector]):
         admin_api_user: str,
         admin_api_password: str,
         default_query_limit: int = 10,
+        vector_precision: Precision = Precision.FLOAT16,
         **extra_params: Any
     ) -> None:
         """
@@ -57,6 +59,7 @@ class MongoDBVectorDatabase(VectorDatabase[MongoDBVDBConnector]):
             admin_api_password (str): The admin API password.
             default_query_limit (int): Default vector search limit,
                 MongoDB does not have a default for it so setting it to a reasonable number of 10.
+            vector_precision (Precision): Precision to use for storing vectors. Defaults to FLOAT16.
             **extra_params (Any): Additional parameters for the MongoDB connection.
         """
         super().__init__()
@@ -66,7 +69,7 @@ class MongoDBVectorDatabase(VectorDatabase[MongoDBVDBConnector]):
             MongoDBAdminParams(cluster_name, project_id, admin_api_user, admin_api_password),
             **extra_params
         )
-        self._settings = VDBSettings(default_query_limit)
+        self._settings = VDBSettings(default_query_limit, vector_precision=vector_precision)
 
     @property
     def _vdb_connector(self) -> MongoDBVDBConnector:
