@@ -27,10 +27,14 @@ from superlinked.framework.common.exception import (
 )
 from superlinked.framework.common.interface.has_length import HasLength
 from superlinked.framework.common.interface.weighted import Weighted
+from superlinked.framework.common.space.embedding.model_based.singleton_embedding_engine_manager import (
+    SingletonEmbeddingEngineManager,
+)
 from superlinked.framework.common.storage_manager.storage_manager import StorageManager
 from superlinked.framework.common.transform.transformation_factory import (
     TransformationFactory,
 )
+from superlinked.framework.common.util.async_util import AsyncUtil
 from superlinked.framework.online.dag.default_online_node import DefaultOnlineNode
 from superlinked.framework.online.dag.evaluation_result import SingleEvaluationResult
 from superlinked.framework.online.dag.online_node import OnlineNode
@@ -52,7 +56,7 @@ class OnlineAggregationNode(DefaultOnlineNode[AggregationNode, Vector], HasLengt
         )
         OnlineAggregationNode._validate_parents(parents)
         self._aggregation_transformation = TransformationFactory.create_aggregation_transformation(
-            self.node.transformation_config,
+            self.node.transformation_config, SingletonEmbeddingEngineManager()
         )
 
     @property
@@ -82,9 +86,11 @@ class OnlineAggregationNode(DefaultOnlineNode[AggregationNode, Vector], HasLengt
         not_empty_weighted_vectors = self._get_not_empty_weighted_vectors(list(parent_results.values()))
         if self._no_event_present(not_empty_weighted_vectors):
             return not_empty_weighted_vectors[0].item
-        return self._aggregation_transformation.transform(
-            not_empty_weighted_vectors,
-            context,
+        return AsyncUtil.run(
+            self._aggregation_transformation.transform(
+                not_empty_weighted_vectors,
+                context,
+            )
         )
 
     def _check_evaluation_inputs(
