@@ -24,11 +24,15 @@ from superlinked.framework.common.dag.context import ExecutionContext
 from superlinked.framework.common.data_types import Vector
 from superlinked.framework.common.interface.has_length import HasLength
 from superlinked.framework.common.parser.parsed_schema import ParsedSchema
+from superlinked.framework.common.space.embedding.model_based.singleton_embedding_engine_manager import (
+    SingletonEmbeddingEngineManager,
+)
 from superlinked.framework.common.storage_manager.storage_manager import StorageManager
 from superlinked.framework.common.transform.transform import Step
 from superlinked.framework.common.transform.transformation_factory import (
     TransformationFactory,
 )
+from superlinked.framework.common.util.async_util import AsyncUtil
 from superlinked.framework.online.dag.evaluation_result import EvaluationResult
 from superlinked.framework.online.dag.online_node import OnlineNode
 
@@ -42,7 +46,7 @@ class OnlineCategoricalSimilarityNode(OnlineNode[CategoricalSimilarityNode, Vect
     ) -> None:
         super().__init__(node, parents, storage_manager)
         self._embedding_transformation = TransformationFactory.create_embedding_transformation(
-            self.node.transformation_config
+            self.node.transformation_config, SingletonEmbeddingEngineManager()
         )
 
     @property
@@ -79,4 +83,4 @@ class OnlineCategoricalSimilarityNode(OnlineNode[CategoricalSimilarityNode, Vect
             return Vector.init_zero_vector(self.node.length)
         input_ = parent_result.main.value
         categories = input_ if isinstance(input_, list) else [input_]
-        return self.embedding_transformation.transform(categories, context)
+        return AsyncUtil.run(self.embedding_transformation.transform(categories, context))
