@@ -28,6 +28,7 @@ from superlinked.framework.common.transform.transform import Step
 from superlinked.framework.common.transform.transformation_factory import (
     TransformationFactory,
 )
+from superlinked.framework.common.util.async_util import AsyncUtil
 from superlinked.framework.online.dag.evaluation_result import EvaluationResult
 from superlinked.framework.online.dag.online_node import OnlineNode
 
@@ -78,7 +79,7 @@ class OnlineCustomVectorEmbeddingNode(OnlineNode[CustomVectorEmbeddingNode, Vect
             return Vector.init_zero_vector(self.node.length)
         input_value = parent_result.main.value
         self._validate_input_value(input_value)
-        return self.embedding_transformation.transform(Vector(input_value), context)
+        return AsyncUtil.run(self.embedding_transformation.transform(Vector(input_value), context))
 
     def _validate_input_value(self, input_value: Sequence[float]) -> None:
         if len(input_value) != self.length:
