@@ -15,10 +15,9 @@
 from __future__ import annotations
 
 from abc import ABC, ABCMeta, abstractmethod
-from functools import partial
 
 import structlog
-from beartype.typing import Generic, Mapping, Sequence
+from beartype.typing import Generic, Mapping, Sequence, cast
 
 from superlinked.framework.common.dag.context import ExecutionContext
 from superlinked.framework.common.dag.exception import ParentCountException
@@ -26,7 +25,10 @@ from superlinked.framework.common.dag.node import NT, Node, NodeDataT
 from superlinked.framework.common.exception import DagEvaluationException
 from superlinked.framework.common.parser.parsed_schema import ParsedSchema
 from superlinked.framework.common.schema.schema_object import SchemaObject
+from superlinked.framework.common.settings import settings
+from superlinked.framework.common.storage_manager.node_result_data import NodeResultData
 from superlinked.framework.common.storage_manager.storage_manager import StorageManager
+from superlinked.framework.common.util.concurrent_executor import ConcurrentExecutor
 from superlinked.framework.online.dag.evaluation_result import (
     EvaluationResult,
     SingleEvaluationResult,
@@ -82,11 +84,11 @@ class OnlineNode(ABC, Generic[NT, NodeDataT], metaclass=ABCMeta):
         parsed_schemas: Sequence[ParsedSchema],
         context: ExecutionContext,
     ) -> list[EvaluationResult[NodeDataT] | None]:
-        results = self.evaluate_self(parsed_schemas, context)
-        if self.node.persist_evaluation_result:
-            for i, result in enumerate(results):
-                if result is not None:
-                    self.persist(result, parsed_schemas[i])
+        with context.dag_output_recorder.record_evaluation_exception(self.node_id):
+            results = self.evaluate_self(parsed_schemas, context)
+            if self.node.persist_node_result:
+                self.persist(results, parsed_schemas)
+        context.dag_output_recorder.record(self.node_id, results)
         return results
 
     def evaluate_next_single(
@@ -105,12 +107,22 @@ class OnlineNode(ABC, Generic[NT, NodeDataT], metaclass=ABCMeta):
     def evaluate_parents(
         self, parents: Sequence[OnlineNode], parsed_schemas: Sequence[ParsedSchema], context: ExecutionContext
     ) -> list[dict[OnlineNode, EvaluationResult]]:
-        inverse_parent_results = {parent: parent.evaluate_next(parsed_schemas, context) for parent in parents}
+        inverse_parent_results = self._evaluate_parents_concurrently(parents, parsed_schemas, context)
         parents_results = [
             {parent: inverse_parent_results[parent][i] for parent in parents} for i in range(len(parsed_schemas))
         ]
         return [self._validate_parents_result(parents_result) for parents_result in parents_results]
 
+    def _evaluate_parents_concurrently(
+        self, parents: Sequence[OnlineNode], parsed_schemas: Sequence[ParsedSchema], context: ExecutionContext
+    ) -> dict[OnlineNode, list[EvaluationResult | None]]:
+        parent_results = ConcurrentExecutor().execute(
+            lambda parent: parent.evaluate_next(parsed_schemas, context),
+            args_list=[(parent,) for parent in parents],
+            condition=settings.SUPERLINKED_CONCURRENT_ONLINE_DAG_EVALUATION,
+        )
+        return dict(zip(parents, parent_results))
+
     def _validate_parents_result(
         self, parents_result: Mapping[OnlineNode, EvaluationResult | None]
     ) -> dict[OnlineNode, EvaluationResult]:
@@ -132,38 +144,49 @@ class OnlineNode(ABC, Generic[NT, NodeDataT], metaclass=ABCMeta):
 
     def persist(
         self,
-        result: EvaluationResult[NodeDataT],
-        parsed_schema: ParsedSchema,
+        results: Sequence[EvaluationResult[NodeDataT] | None],
+        parsed_schemas: Sequence[ParsedSchema],
     ) -> None:
-        self.storage_manager.write_node_result(
-            parsed_schema.schema,
-            parsed_schema.id_,
-            result.main.node_id,
-            result.main.value,
-        )
-        for chunk in result.chunks:
-            self.storage_manager.write_node_result(
-                parsed_schema.schema,
-                chunk.object_id,
-                chunk.node_id,
-                chunk.value,
-                parsed_schema.id_,
-            )
+        node_data_items = [
+            data
+            for i, result in enumerate(results)
+            if result is not None
+            for data in [
+                NodeResultData(
+                    parsed_schemas[i].schema._schema_name, parsed_schemas[i].id_, result.main.node_id, result.main.value
+                ),
+                *[
+                    NodeResultData(
+                        parsed_schemas[i].schema._schema_name,
+                        chunk.object_id,
+                        chunk.node_id,
+                        chunk.value,
+                        parsed_schemas[i].id_,
+                    )
+                    for chunk in result.chunks
+                ],
+            ]
+        ]
+        self.storage_manager.write_node_results(node_data_items)
         logger.debug(
             "stored online node data",
-            schema=parsed_schema.schema._schema_name,
-            pii_main_result=partial(str, result.main.value),
-            pii_chunk_result=lambda: [str(chunk.value) for chunk in result.chunks],
+            schemas=lambda: {parsed_schema.schema._schema_name for parsed_schema in parsed_schemas},
+            n_results=len(node_data_items),
         )
 
     def load_stored_results(
         self, schemas_with_object_ids: Sequence[tuple[SchemaObject, str]]
     ) -> list[NodeDataT | None]:
-        return self.storage_manager.read_node_results(
-            schemas_with_object_ids,
+        if not schemas_with_object_ids:
+            return []
+        distinct_keys = list(set(schemas_with_object_ids))
+        distinct_stored_results = self.storage_manager.read_node_results(
+            distinct_keys,
             self.node_id,
             self.node.node_data_type,
         )
+        distinct_stored_result_by_key = dict(zip(distinct_keys, distinct_stored_results))
+        return [distinct_stored_result_by_key[key] for key in schemas_with_object_ids]
 
     def load_stored_results_with_default(
         self, schemas_with_object_ids: Sequence[tuple[SchemaObject, str]], default_value: NodeDataT
@@ -172,24 +195,27 @@ class OnlineNode(ABC, Generic[NT, NodeDataT], metaclass=ABCMeta):
             default_value if result is None else result for result in self.load_stored_results(schemas_with_object_ids)
         ]
 
-    def load_stored_result(self, schema: SchemaObject, object_id: str) -> NodeDataT | None:
-        return self.load_stored_results([(schema, object_id)])[0]
-
-    def load_stored_result_or_raise_exception(
+    def load_stored_results_or_raise_exception(
         self,
-        parsed_schema: ParsedSchema,
-    ) -> NodeDataT:
-        stored_result = self.load_stored_result(parsed_schema.schema, parsed_schema.id_)
-        if stored_result is None:
+        parsed_schemas: Sequence[ParsedSchema],
+    ) -> list[NodeDataT]:
+        schemas_with_object_ids = [(parsed_schema.schema, parsed_schema.id_) for parsed_schema in parsed_schemas]
+        stored_results = self.load_stored_results(schemas_with_object_ids)
+        if none_indices := [i for i, stored_result in enumerate(stored_results) if stored_result is None]:
+            wrong_parsed_schema_params = [
+                f"{parsed_schemas[index].schema._schema_name}, {parsed_schemas[index].id_}" for index in none_indices
+            ]
             raise DagEvaluationException(
-                f"{self.node_id} doesn't have a stored value for (schema, object_id):"
-                + f" ({parsed_schema.schema._schema_name}, {parsed_schema.id_})"
+                f"{self.node_id} doesn't have stored values for the following (schema, object_id) pairs:"
+                + f" ({wrong_parsed_schema_params})"
             )
-        return stored_result
+        return cast(list[NodeDataT], stored_results)
 
     def validate_parents(
         self,
         parent_validation_type: ParentValidationType,
     ) -> None:
         if not parent_validation_type.validator(len(self.parents)):
-            raise ParentCountException(f"{type(self).__name__} must have {parent_validation_type.description}.")
+            raise ParentCountException(
+                f"{type(self).__name__} must have {parent_validation_type.description}, got {len(self.parents)}"
+            )
