@@ -22,11 +22,15 @@ from superlinked.framework.common.dag.text_embedding_node import TextEmbeddingNo
 from superlinked.framework.common.data_types import Vector
 from superlinked.framework.common.interface.has_length import HasLength
 from superlinked.framework.common.parser.parsed_schema import ParsedSchema
+from superlinked.framework.common.space.embedding.model_based.singleton_embedding_engine_manager import (
+    SingletonEmbeddingEngineManager,
+)
 from superlinked.framework.common.storage_manager.storage_manager import StorageManager
 from superlinked.framework.common.transform.transform import Step
 from superlinked.framework.common.transform.transformation_factory import (
     TransformationFactory,
 )
+from superlinked.framework.common.util.async_util import AsyncUtil
 from superlinked.framework.online.dag.default_online_node import DefaultOnlineNode
 from superlinked.framework.online.dag.evaluation_result import SingleEvaluationResult
 from superlinked.framework.online.dag.online_node import OnlineNode
@@ -43,7 +47,9 @@ class OnlineTextEmbeddingNode(DefaultOnlineNode[TextEmbeddingNode, Vector], HasL
         self._embedding_transformation = self._init_embedding_transformation()
 
     def _init_embedding_transformation(self) -> Step[Sequence[str], list[Vector]]:
-        return TransformationFactory.create_multi_embedding_transformation(self.node.transformation_config)
+        return TransformationFactory.create_multi_embedding_transformation(
+            self.node.transformation_config, SingletonEmbeddingEngineManager()
+        )
 
     @property
     @override
@@ -55,11 +61,16 @@ class OnlineTextEmbeddingNode(DefaultOnlineNode[TextEmbeddingNode, Vector], HasL
         return self._embedding_transformation
 
     @override
-    def get_fallback_result(self, parsed_schema: ParsedSchema) -> Vector:
-        stored_result = self.load_stored_result(parsed_schema.schema, parsed_schema.id_)
-        if stored_result is not None:
-            return stored_result
-        return Vector.init_zero_vector(self.node.length)
+    def get_fallback_results(
+        self,
+        parsed_schemas: Sequence[ParsedSchema],
+    ) -> list[Vector]:
+        schemas_with_object_ids = [(parsed_schema.schema, parsed_schema.id_) for parsed_schema in parsed_schemas]
+        stored_results = self.load_stored_results(schemas_with_object_ids)
+        return [
+            Vector.init_zero_vector(self.node.length) if stored_result is None else stored_result
+            for stored_result in stored_results
+        ]
 
     @override
     def _evaluate_singles(
@@ -81,4 +92,4 @@ class OnlineTextEmbeddingNode(DefaultOnlineNode[TextEmbeddingNode, Vector], HasL
         return embedded_texts
 
     def __embed_texts(self, texts: Sequence[str], context: ExecutionContext) -> list[Vector]:
-        return self.embedding_transformation.transform(texts, context)
+        return AsyncUtil.run(self.embedding_transformation.transform(texts, context))
