@@ -26,6 +26,9 @@ from superlinked.framework.common.storage.entity.entity_data import EntityData
 from superlinked.framework.common.storage.entity.entity_id import EntityId
 from superlinked.framework.common.storage.field.field import Field
 from superlinked.framework.common.storage.field.field_data import FieldData
+from superlinked.framework.common.storage.query.vdb_knn_search_config import (
+    VDBKNNSearchConfig,
+)
 from superlinked.framework.common.storage.query.vdb_knn_search_params import (
     VDBKNNSearchParams,
 )
@@ -34,6 +37,7 @@ from superlinked.framework.common.storage.search_index.manager.search_index_mana
     SearchIndexManager,
 )
 from superlinked.framework.common.storage.vdb_connector import VDBConnector
+from superlinked.framework.dsl.query.query_user_config import QueryUserConfig
 from superlinked.framework.storage.common.vdb_settings import VDBSettings
 from superlinked.framework.storage.in_memory.in_memory_search import InMemorySearch
 from superlinked.framework.storage.in_memory.in_memory_search_index_manager import (
@@ -43,16 +47,15 @@ from superlinked.framework.storage.in_memory.json_codec import JsonDecoder, Json
 from superlinked.framework.storage.in_memory.object_serializer import ObjectSerializer
 
 
-class InMemoryVDB(VDBConnector):
+class InMemoryVDB(VDBConnector[VDBKNNSearchConfig]):
     def __init__(self, vdb_settings: VDBSettings) -> None:
-        super().__init__()
+        super().__init__(vdb_settings=vdb_settings)
         self._vdb = defaultdict[str, dict[str, Any]](dict)
         self._search = InMemorySearch()
-        self.__vdb_settings = vdb_settings
         self.__search_index_manager = InMemorySearchIndexManager()
 
     @override
-    def close_connection(self) -> None:
+    async def close_connection(self) -> None:
         self._vdb = defaultdict[str, dict[str, Any]](dict)
         self.search_index_manager.clear_configs()
 
@@ -61,19 +64,14 @@ class InMemoryVDB(VDBConnector):
     def search_index_manager(self) -> SearchIndexManager:
         return self.__search_index_manager
 
-    @property
     @override
-    def _default_search_limit(self) -> int:
-        return self.__vdb_settings.default_query_limit
-
-    @override
-    def write_entities(self, entity_data: Sequence[EntityData]) -> None:
+    async def write_entities(self, entity_data: Sequence[EntityData]) -> None:
         for ed in entity_data:
             row_id = InMemoryVDB._get_row_id_from_entity_id(ed.id_)
             self._vdb[row_id].update({name: fd.value for name, fd in ed.field_data.items()})
 
     @override
-    def read_entities(self, entities: Sequence[Entity]) -> Sequence[EntityData]:
+    async def read_entities(self, entities: Sequence[Entity]) -> Sequence[EntityData]:
         return [
             EntityData(
                 entity.id_,
@@ -109,20 +107,25 @@ class InMemoryVDB(VDBConnector):
         ]
 
     @override
-    def _knn_search(
+    async def _knn_search(
         self,
         index_name: str,
         schema_name: str,
         vdb_knn_search_params: VDBKNNSearchParams,
+        search_config: VDBKNNSearchConfig,
         **params: Any,
     ) -> Sequence[ResultEntityData]:
         index_config = self._get_index_config(index_name)
-        sorted_scores = self._search.knn_search(index_config, self._vdb, vdb_knn_search_params)
+        sorted_scores = await self._search.knn_search(index_config, self._vdb, vdb_knn_search_params)
         return [
             self._get_result_entity_data(row_id, score, vdb_knn_search_params.fields_to_return)
             for row_id, score in sorted_scores
         ]
 
+    @override
+    def init_search_config(self, query_user_config: QueryUserConfig) -> VDBKNNSearchConfig:
+        return VDBKNNSearchConfig()
+
     @override
     def persist(self, serializer: ObjectSerializer) -> None:
         app_identifier = "_".join(self.search_index_manager._index_configs.keys())
