@@ -18,6 +18,7 @@ import numpy as np
 from beartype.typing import Any, Callable, Sequence
 
 from superlinked.framework.common.data_types import Json, Vector
+from superlinked.framework.common.precision import Precision
 from superlinked.framework.common.schema.blob_information import BlobInformation
 from superlinked.framework.common.storage.exception import EncoderException
 from superlinked.framework.common.storage.field.field import Field
@@ -28,14 +29,16 @@ MongoDBEncodedTypes = str | float | int | Sequence[float]
 
 
 class MongoDBFieldEncoder:
-    def __init__(self) -> None:
+    def __init__(self, vector_precision: Precision) -> None:
         self._encode_map: dict[FieldDataType, Callable[..., Any]] = {
             FieldDataType.BLOB: self._encode_blob,
             FieldDataType.DOUBLE: self._encode_double,
             FieldDataType.FLOAT_LIST: self._encode_float_list,
             FieldDataType.INT: self._encode_int,
+            FieldDataType.BOOLEAN: self._encode_boolean,
             FieldDataType.JSON: self._encode_json,
             FieldDataType.STRING: self._encode_string,
+            FieldDataType.METADATA_STRING: self._encode_string,
             FieldDataType.STRING_LIST: self._encode_string_list,
             FieldDataType.VECTOR: self._encode_vector,
         }
@@ -44,11 +47,14 @@ class MongoDBFieldEncoder:
             FieldDataType.DOUBLE: self._decode_double,
             FieldDataType.FLOAT_LIST: self._decode_float_list,
             FieldDataType.INT: self._decode_int,
+            FieldDataType.BOOLEAN: self._decode_boolean,
             FieldDataType.JSON: self._decode_json,
             FieldDataType.STRING: self._decode_string,
+            FieldDataType.METADATA_STRING: self._encode_string,
             FieldDataType.STRING_LIST: self._decode_string_list,
             FieldDataType.VECTOR: self._decode_vector,
         }
+        self.__vector_precision_type = vector_precision.to_np_type()
 
     def _encode_blob(self, blob: BlobInformation) -> str | None:
         return blob.path
@@ -74,6 +80,12 @@ class MongoDBFieldEncoder:
     def _decode_int(self, int_: int) -> int:
         return int_
 
+    def _encode_boolean(self, bool_: bool) -> bool:
+        return bool_
+
+    def _decode_boolean(self, bool_: bool) -> bool:
+        return bool_
+
     def _encode_json(self, json_: Json) -> str:
         return json.dumps(json_, ensure_ascii=True)
 
@@ -93,12 +105,12 @@ class MongoDBFieldEncoder:
         return string_list
 
     def _encode_vector(self, vector: Vector) -> Sequence[float]:
-        return vector.value.astype(np.float32).tolist()
+        return vector.value.astype(self.__vector_precision_type).tolist()
 
     def _decode_vector(self, vector: Any) -> Vector:
         if not isinstance(vector, list):
             raise NotImplementedError(f"Cannot decode non-list type vector, got: {type(vector)}")
-        return Vector(np.array(vector).astype(np.float32).tolist())
+        return Vector(np.array(vector).astype(self.__vector_precision_type).tolist())
 
     def encode_field(self, field: FieldData) -> MongoDBEncodedTypes:
         if encoder := self._encode_map.get(field.data_type):
