@@ -11,6 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import re
 
 from beartype.typing import Any, Sequence, cast
 from qdrant_client import QdrantClient
@@ -38,6 +39,9 @@ from superlinked.framework.common.storage.field.field import Field
 from superlinked.framework.common.storage.field.field_data import FieldData
 from superlinked.framework.common.storage.field.field_data_type import FieldDataType
 from superlinked.framework.common.storage.index_config import IndexConfig
+from superlinked.framework.common.storage.query.vdb_knn_search_config import (
+    VDBKNNSearchConfig,
+)
 from superlinked.framework.common.storage.query.vdb_knn_search_params import (
     VDBKNNSearchParams,
 )
@@ -47,6 +51,7 @@ from superlinked.framework.common.storage.search_index.manager.search_index_mana
 )
 from superlinked.framework.common.storage.vdb_connector import VDBConnector
 from superlinked.framework.common.util.string_util import StringUtil
+from superlinked.framework.dsl.query.query_user_config import QueryUserConfig
 from superlinked.framework.storage.common.vdb_settings import VDBSettings
 from superlinked.framework.storage.qdrant.qdrant_connection_params import (
     QdrantConnectionParams,
@@ -67,21 +72,22 @@ ID_PAYLOAD_FIELD_NAME = "__original_entity_id__"
 ID_PAYLOAD_FIELD = Field(FieldDataType.STRING, ID_PAYLOAD_FIELD_NAME)
 
 
-class QdrantVDBConnector(VDBConnector):
+class QdrantVDBConnector(VDBConnector[VDBKNNSearchConfig]):
     def __init__(self, connection_params: QdrantConnectionParams, vdb_settings: VDBSettings) -> None:
-        super().__init__()
+        super().__init__(vdb_settings=vdb_settings)
         self._client = QdrantClient(
             url=connection_params.connection_string,
             api_key=connection_params._api_key,
+            timeout=connection_params.timeout,
+            prefer_grpc=connection_params.prefer_grpc,
         )
-        self._encoder = QdrantFieldEncoder()
+        self._encoder = QdrantFieldEncoder(self.vector_precision)
         self.__search_index_manager = QdrantSearchIndexManager(self._client)
         self._search = QdrantSearch(self._client, self._encoder)
         self._vector_field_names = list[str]()
-        self.__vdb_settings = vdb_settings
 
     @override
-    def close_connection(self) -> None:
+    async def close_connection(self) -> None:
         self._client.close()
 
     @property
@@ -89,11 +95,6 @@ class QdrantVDBConnector(VDBConnector):
     def search_index_manager(self) -> SearchIndexManager:
         return self.__search_index_manager
 
-    @property
-    @override
-    def _default_search_limit(self) -> int:
-        return self.__vdb_settings.default_query_limit
-
     @override
     def init_search_index_configs(
         self,
@@ -107,7 +108,7 @@ class QdrantVDBConnector(VDBConnector):
         )
 
     @override
-    def write_entities(self, entity_data: Sequence[EntityData]) -> None:
+    async def write_entities(self, entity_data: Sequence[EntityData]) -> None:
         if not self.search_index_manager._index_configs:
             raise IndexConfigNotFoundException(
                 f"{type(self).__name__} can work properly only after initializing " + "the search indices."
@@ -170,61 +171,83 @@ class QdrantVDBConnector(VDBConnector):
         return non_existing_points, existing_points
 
     @override
-    def read_entities(self, entities: Sequence[Entity]) -> Sequence[EntityData]:
+    async def read_entities(self, entities: Sequence[Entity]) -> Sequence[EntityData]:
         returned_field_names = {field.name for entity in entities for field in entity.fields.values()} | {
             ID_PAYLOAD_FIELD_NAME
         }
         vector_fields = list(
             {field_name for field_name in returned_field_names if field_name in self._vector_field_names}
         )
-        payload_fields = list({field_name for field_name in returned_field_names if field_name not in vector_fields})
+        payload_fields = list(
+            {
+                self.sanitize_field_name(field_name)
+                for field_name in returned_field_names
+                if field_name not in vector_fields
+            }
+        )
         points = self._client.retrieve(
             self.collection_name,
             ids=[QdrantVDBConnector._get_qdrant_id(entity.id_) for entity in entities],
             with_vectors=vector_fields or False,
             with_payload=payload_fields or False,
         )
-        entity_by_id = {entity.id_: entity for entity in entities}
-        return [self._get_entity_data_from_point(point, entity_by_id) for point in points]
+        points_by_entity_id = {
+            self._entity_id_from_payload(payload): point
+            for point in points
+            if (payload := self._check_and_get_payload(point))
+        }
+        return [self._get_entity_data_from_point(entity, points_by_entity_id.get(entity.id_)) for entity in entities]
 
-    def _get_entity_data_from_point(self, point: Record, entity_by_id: dict[EntityId, Entity]) -> EntityData:
+    def _get_entity_data_from_point(self, entity: Entity, point: Record | None) -> EntityData:
+        if point is None:
+            return EntityData(entity.id_, {})
         payload_fields = self._check_and_get_payload(point)
         vector_fields = self._check_and_get_vectors(point)
         all_fields_to_return = payload_fields | vector_fields
-        id_ = self._entity_id_from_payload(payload_fields)
-        fields = entity_by_id[id_].fields
         return EntityData(
-            id_,
+            entity.id_,
             {
-                field.name: self._encoder.decode_field(field, all_fields_to_return.get(field.name))
-                for field in fields.values()
+                field.name: self._encoder.decode_field(field, field_value)
+                for field, field_value in [
+                    (field, all_fields_to_return.get(field.name)) for field in entity.fields.values()
+                ]
+                if field_value is not None
             },
         )
 
     @override
-    def _knn_search(
+    async def _knn_search(
         self,
         index_name: str,
         schema_name: str,
         vdb_knn_search_params: VDBKNNSearchParams,
+        search_config: VDBKNNSearchConfig,
         **params: Any,
     ) -> Sequence[ResultEntityData]:
         index_config = self._get_index_config(index_name)
         extended_fields_to_return = list(vdb_knn_search_params.fields_to_return) + [ID_PAYLOAD_FIELD]
-        result: QueryResponse = self._search.knn_search_with_checks(
+        result: QueryResponse = await self._search.knn_search_with_checks(
             index_config,
             QdrantVDBKNNSearchParams.from_base(vdb_knn_search_params, self.collection_name, extended_fields_to_return),
+            search_config,
         )
         return [
             self._get_result_entity_data_from_point(point, vdb_knn_search_params.fields_to_return)
             for point in self._calculate_sorted_result_points(result.points)
         ]
 
+    @override
+    def init_search_config(self, query_user_config: QueryUserConfig) -> VDBKNNSearchConfig:
+        return VDBKNNSearchConfig()
+
     def _calculate_sorted_result_points(self, scored_points: Sequence[ScoredPoint]) -> list[ScoredPoint]:
         """Sort by score descending, then by id ascending"""
         result_points = list(scored_points)
         result_points.sort(
-            key=lambda point: (-point.score, point.payload.get(ID_PAYLOAD_FIELD_NAME) if point.payload else None)
+            key=lambda point: (
+                -point.score,
+                point.payload.get(ID_PAYLOAD_FIELD_NAME) if point.payload else None,
+            )
         )
         return result_points
 
@@ -250,7 +273,7 @@ class QdrantVDBConnector(VDBConnector):
             return {}
         if not isinstance(point.vector, dict) or any(v for v in point.vector.values() if not isinstance(v, list)):
             raise ValueError(f"Retrieved point's payload is invalid: {[point.payload]}")
-        return cast(dict[str, list[float]], point.payload)
+        return cast(dict[str, list[float]], point.vector)
 
     def _check_and_get_payload(self, point: Record | ScoredPoint) -> dict[str, Any]:
         if point.payload is None or not isinstance(point.payload, dict):
@@ -271,3 +294,7 @@ class QdrantVDBConnector(VDBConnector):
             raise ValueError(f"The mandatory {ID_PAYLOAD_FIELD_NAME} is invalid: {id_str}.")
         schema_id, object_id = self._encoder._decode_base_type(id_str).split(":", 1)
         return EntityId(schema_id=schema_id, object_id=object_id)
+
+    @staticmethod
+    def sanitize_field_name(field_name: str) -> str:
+        return re.sub(r"[^A-Za-z0-9_]", "_", field_name)
