@@ -14,36 +14,40 @@
 
 
 from beartype.typing import Any, cast
-from redis import Redis
 from typing_extensions import override
 
+from superlinked.framework.common.precision import Precision
 from superlinked.framework.common.storage.index_config import IndexConfig
 from superlinked.framework.common.storage.query.vdb_knn_search_params import (
     VDBKNNSearchParams,
 )
 from superlinked.framework.common.storage.search import Search
 from superlinked.framework.storage.redis.query.redis_query_builder import (
-    RedisQuery,
     RedisQueryBuilder,
+    VectorQueryObj,
+)
+from superlinked.framework.storage.redis.query.vdb_knn_search_params import (
+    RedisVDBKNNSearchConfig,
 )
 from superlinked.framework.storage.redis.redis_field_encoder import RedisFieldEncoder
+from superlinked.framework.storage.redis.redis_vdb_client import RedisVDBClient
 
 
-class RedisSearch(Search[VDBKNNSearchParams, RedisQuery, dict[bytes, Any]]):
-    def __init__(self, client: Redis, encoder: RedisFieldEncoder) -> None:
+class RedisSearch(Search[VDBKNNSearchParams, VectorQueryObj, dict[bytes, Any], RedisVDBKNNSearchConfig]):
+    def __init__(self, client: RedisVDBClient, encoder: RedisFieldEncoder, vector_precision: Precision) -> None:
         super().__init__()
         self._client = client
-        self._query_builder = RedisQueryBuilder(encoder)
+        self._query_builder = RedisQueryBuilder(encoder, vector_precision)
 
     @override
-    def build_query(self, search_params: VDBKNNSearchParams) -> RedisQuery:
-        return self._query_builder.build_query(search_params)
+    def build_query(self, search_params: VDBKNNSearchParams, search_config: RedisVDBKNNSearchConfig) -> VectorQueryObj:
+        return self._query_builder.build_query(search_params, search_config)
 
     @override
-    def knn_search(
+    async def knn_search(
         self,
         index_config: IndexConfig,
-        query: RedisQuery,
+        query: VectorQueryObj,
     ) -> dict[bytes, Any]:
-        result = self._client.ft(index_config.index_name).search(query.query, query_params=query.query_params)
+        result = await self._client.client.ft(index_config.index_name).search(query.query, query.params)
         return cast(dict[bytes, Any], result)
