@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import redis
+import structlog
 from beartype.typing import Any, Sequence
 from typing_extensions import override
 
@@ -29,9 +29,17 @@ from superlinked.framework.common.storage.search_index.manager.search_index_mana
     SearchIndexManager,
 )
 from superlinked.framework.common.storage.vdb_connector import VDBConnector
+from superlinked.framework.dsl.query.query_user_config import QueryUserConfig
 from superlinked.framework.storage.common.vdb_settings import VDBSettings
-from superlinked.framework.storage.redis.query.redis_query_builder import (
-    VECTOR_DISTANCE_ALIAS,
+from superlinked.framework.storage.redis.exception import (
+    RedisResultException,
+    RedisTimeoutException,
+)
+from superlinked.framework.storage.redis.query.redis_vector_query_params import (
+    DISTANCE_ID,
+)
+from superlinked.framework.storage.redis.query.vdb_knn_search_params import (
+    RedisVDBKNNSearchConfig,
 )
 from superlinked.framework.storage.redis.redis_connection_params import (
     RedisConnectionParams,
@@ -41,35 +49,37 @@ from superlinked.framework.storage.redis.redis_search import RedisSearch
 from superlinked.framework.storage.redis.redis_search_index_manager import (
     RedisSearchIndexManager,
 )
+from superlinked.framework.storage.redis.redis_vdb_client import RedisVDBClient
+
+logger = structlog.getLogger()
+UNAFFECTING_DISTANCE = 0.0
 
 
-class RedisVDBConnector(VDBConnector):
-    def __init__(self, connection_params: RedisConnectionParams, vdb_settings: VDBSettings) -> None:
-        super().__init__()
-        self._client: redis.Redis = redis.from_url(connection_params.connection_string, protocol=3)
-        self._encoder = RedisFieldEncoder()
+class RedisVDBConnector(VDBConnector[RedisVDBKNNSearchConfig]):
+    def __init__(
+        self,
+        connection_params: RedisConnectionParams,
+        vdb_settings: VDBSettings,
+    ) -> None:
+        super().__init__(vdb_settings=vdb_settings)
+        self._client = RedisVDBClient(connection_params.connection_string)
+        self._encoder = RedisFieldEncoder(self.vector_precision)
         self.__search_index_manager = RedisSearchIndexManager(self._client, self._encoder)
-        self._search = RedisSearch(self._client, self._encoder)
-        self.__vdb_settings = vdb_settings
+        self._search = RedisSearch(self._client, self._encoder, self.vector_precision)
 
     @override
-    def close_connection(self) -> None:
-        self._client.close()
+    async def close_connection(self) -> None:
+        await self._client.client.close()
 
     @property
     @override
     def search_index_manager(self) -> SearchIndexManager:
         return self.__search_index_manager
 
-    @property
     @override
-    def _default_search_limit(self) -> int:
-        return self.__vdb_settings.default_query_limit
-
-    @override
-    def write_entities(self, entity_data: Sequence[EntityData]) -> None:
+    async def write_entities(self, entity_data: Sequence[EntityData]) -> None:
         if entity_data:
-            _pipeline = self._client.pipeline(transaction=False)
+            _pipeline = self._client.client.pipeline(transaction=False)
             for ed in entity_data:
                 if ed.field_data:
                     _pipeline.hset(
@@ -78,15 +88,15 @@ class RedisVDBConnector(VDBConnector):
                             field_name: self._encoder.encode_field(field) for field_name, field in ed.field_data.items()
                         },
                     )
-            _pipeline.execute()
+            await _pipeline.execute()
 
     @override
-    def read_entities(self, entities: Sequence[Entity]) -> Sequence[EntityData]:
+    async def read_entities(self, entities: Sequence[Entity]) -> Sequence[EntityData]:
         valid_entities = [entity for entity in entities if entity.fields]
         if not valid_entities:
             return []
 
-        pipeline = self._client.pipeline(transaction=False)
+        pipeline = self._client.client.pipeline(transaction=False)
 
         for entity in valid_entities:
             pipeline.hmget(
@@ -94,7 +104,7 @@ class RedisVDBConnector(VDBConnector):
                 [field.name for field in entity.fields.values()],
             )
 
-        all_encoded_values = pipeline.execute()
+        all_encoded_values = await pipeline.execute()
 
         return [
             EntityData(
@@ -109,17 +119,27 @@ class RedisVDBConnector(VDBConnector):
         ]
 
     @override
-    def _knn_search(
+    async def _knn_search(
         self,
         index_name: str,
         schema_name: str,
         vdb_knn_search_params: VDBKNNSearchParams,
+        search_config: RedisVDBKNNSearchConfig,
         **params: Any,
     ) -> Sequence[ResultEntityData]:
         index_config = self._get_index_config(index_name)
-        result = self._encoder.convert_bytes_keys_dict(
-            self._search.knn_search_with_checks(index_config, vdb_knn_search_params)
-        )
+        result = await self._search.knn_search_with_checks(index_config, vdb_knn_search_params, search_config)
+
+        if isinstance(result, list):
+            raise RedisTimeoutException()
+
+        encoded_result = self._encoder.convert_bytes_keys_dict(result)
+        result_entities = encoded_result["results"]
+        filtered_result_entities = [result_entity for result_entity in result_entities if result_entity is not None]
+
+        if len(filtered_result_entities) < len(result_entities):
+            raise RedisResultException()
+
         return [
             ResultEntityData(
                 RedisVDBConnector._get_entity_id_from_redis_id(self._encoder._decode_string(document["id"])),
@@ -127,22 +147,32 @@ class RedisVDBConnector(VDBConnector):
                     document["extra_attributes"], vdb_knn_search_params.fields_to_return
                 ),
                 self._convert_distance_to_score(
-                    self._encoder._decode_double(document["extra_attributes"][VECTOR_DISTANCE_ALIAS])
+                    self._encoder._decode_double(
+                        document.get("extra_attributes", {}).get(DISTANCE_ID, UNAFFECTING_DISTANCE)
+                    )
                 ),
             )
-            for document in result["results"]
+            for document in filtered_result_entities
         ]
 
+    @override
+    def init_search_config(self, query_user_config: QueryUserConfig) -> RedisVDBKNNSearchConfig:
+        return RedisVDBKNNSearchConfig(query_user_config.redis_hybrid_policy, query_user_config.redis_batch_size)
+
     def _convert_distance_to_score(self, distance: float) -> float:
         return 1 - distance
 
     def _extract_fields_from_document(
         self, document: dict[str, Any], fields_to_return: Sequence[Field]
     ) -> dict[str, FieldData]:
+        if not document:
+            raise RedisResultException()
+        if any(field.name not in document for field in fields_to_return):
+            raise RedisResultException()
         return {
-            returned_field.name: self._encoder.decode_field(returned_field, document[returned_field.name])
-            for returned_field in fields_to_return
-            if document.get(returned_field.name) is not None
+            field.name: self._encoder.decode_field(field, document[field.name])
+            for field in fields_to_return
+            if field.name in document
         }
 
     @staticmethod
